; -------------------------------------
; Z80 SRAM K1-bus micro computer board 
; with Return-oriented Virtual Machine 
; (c) 2016 - 2017 kio@little-bat.de
; -------------------------------------


#target rom
#include "settings.s"


; -------------------------------------------
;		K1-BUS SETTINGS
; -------------------------------------------

option_a5	= 1 ; 1 => use A5 for rd=1/wr=0 selection, use A8 for K1-bus A5
		    ; used when use of WR results in spikes on the k1-bus strobe lines

bSIO_SELECT	= 1 ; the SIO board is jumpered for D1
bIDE_SELECT	= 2 ; the IDE board uses D2

mSIO_SELECT	= 0xff - 1<<bSIO_SELECT;
mIDE_SELECT	= 0xff - 1<<bIDE_SELECT;


; -------------------------------------------
; 		MEMORY LAYOUT
; -------------------------------------------

				; $0000 - $007F	  RST vectors, interrupt handler, eeprom helper 
				; $0080 - $00FF	  global variables
user_program 	= $100 		; $0100 - $1FFF	  user program
			    	;
dflt_stacks  	= $2000		; $2000 - $20FF	  VM RStack (via BC')
				; $2100 - $25FF	  VM VStack (machine stack)
dflt_heap	= $2600		; $2600 -	  dynamically allocated memory
				;       - $7EFF	  handles for allocated memory
;sio_buffers	= $7F00		; $7F00 - $7FFF	  SIO buffers
ram_end		= $8000;sio_buffers


; -------------------------------------------
;		K1-BUS CONSTANTS
; -------------------------------------------

k1_wr_sel	equ	$00	; A765=000	select board
k1_wr_irpt	equ	$40	; A765=010	write interrupt enable mask to boards
k1_wr_hi	equ	$80	; A765=100	write byte to lo-to-hi data latch
k1_wr_data	equ	$c0	; A765=110	write data to selected board; A0-A5 select register

#if option_a5 
k1_rd_irpt	equ	$20	; A765=001	read interrupt state bits (0=active)
k1_rd_i2c	equ	$60	; A765=011	read or write data to i2c bus
k1_rd_data	equ	$a0	; A765=101	read data from selected board; A0-A5 select register
k1_rd_hi	equ	$e0	; A765=111	read data from hi-to-low data latch
#else
k1_rd_irpt	equ	$00	; A76=00	read interrupt state bits (0=active)
k1_rd_i2c	equ	$40	; A76=01	read or write data to i2c bus
k1_rd_data	equ	$80	; A76=10	read data from selected board; A0-A5 select register
k1_rd_hi	equ	$c0	; A76=11	read data from hi-to-low data latch
#endif

i2c_d0_c0	equ	k1_rd_i2c	; i2c data=0 clock=0	$08 = A3 = data
i2c_d0_c1	equ	k1_rd_i2c | $10	; i2c data=0 clock=1	$10 = A4 = clock
i2c_d1_c0	equ	k1_rd_i2c | $08	; i2c data=1 clock=0
i2c_d1_c1	equ	k1_rd_i2c | $18	; i2c data=1 clock=1	= idle bus
i2c_idle	equ	i2c_d1_c1
yellow_led	equ	i2c_d1_c0	; yellow led == i2c clock=0  NOTE: avoid setting red without yellow: 
red_led		equ	i2c_d0_c1	; red led    == i2c data=0	   this is the i2c start condition!
both_leds	equ	i2c_d0_c0	; yellow_led + red_led
no_led		equ	i2c_d1_c1

k1_i2c_din	equ	$80		; mask for D7 = i2c data_in bit


; -------------------------------------------
; 		OTHER CONSTANTS
; -------------------------------------------


; ===========================================
; 		MEMORY LAYOUT
; ===========================================

; RAM:
#data 	DATA, page0end

; ROM:
#code 	GSINIT, $8000	; boot code; rom starts at $8000
#code 	PAGE0		; initializer data for RST etc. and memory management
	.phase $0000	; data will be copied to ram at $0000 => set virtual origin for convenience
#code	VM_CORE
#code 	VM_BYTE
#code	VM_INT
#code	VM_LONG
#code	VM_FLOAT
#code	VM_STRINGS
#code	VM_MEMORY
#code 	CODE		; most other code, e.g. SIO handler
#code	CONST		; const data
#code	ROMEND		; for easy calculation




; ===========================================
;	now: adding actual code and data
;	switching between segments as needed
; ===========================================

#code GSINIT
; after power-on the rom is also visible at $0000 so that the Z80 can start here
; we jump immediately to the standard mapping address so we can switch off init mode
reset::	jp	8003$

8003$:	di			; disable interrupts (safety only, the interrupts are *never* enabled)
	im	1		; init interrupt mode (security only)

; reset k1 bus:
	ld	a, 0xff		; the first IORQ also resets the INIT flip flop => ram becomes accessible
	out	k1_wr_sel, a	; => deselect all boards (security only)
	out	k1_wr_irpt, a	; => disable interrupts on k1-bus boards
	in	a, i2c_idle 	; => set i2c bus to idle state, note: switches off the debug LEDs 

; test for SDP write protected eeprom in ram socket:
	ld	h,0		; hl -> ram
	inc	(hl)		; write to ram
	ld	a,(hl)
	xor	(hl)		; 0x00 expected
	cp	0x40		; 0x40 -> D6 toggled -> eeprom
	jp	z,unlock_eeprom_in_ram_socket	

; init sio1 UART early,
; so that the user has enough time to press "F" for firmware download:
	ld	hl,sio1_init_data
	ld	b, sio1_init_data_count
7$:	ld	c,(hl++)
	ld	a,(hl++)
	out	(c),a
	djnz	7$
	in	a,(k1_rd_data+STCR)    	; start C/T 
	in	a,(k1_rd_data+GPR)	; check test value in GPR
	cp	123
	ld	a,ERROR_SIO_NOT_FOUND
	jp	nz,panic

; test & clear ram to $00		
#include "asm/ramtest.s"		; -> hl = ramtop = 0x8000
	ld	sp,dflt_heap

; copy rst vectors, interrupt vectors and other initialized global variables:
	ld	hl, PAGE0
	ld	de, 0
	ld	bc, page0end - 0
	ldir

; SDP write-protect eeprom:
	call	lock_eeprom
	jr	c,11$			; CY -> error -> not an eeprom
	ld	hl,sysflags
	inc	(hl)			; OK -> set bEEPROM in sysflags

; download firmware if uppercase "F" was received on sioA:
11$:	;ld	a,mSIO_SELECT		; sio2 still selected from above
	;out	(k1_wr_sel),a
	call	sioA_test_char
	jr	z,12$			; no key pressed
	cp	a,"F"
	jr	nz,12$			; no "F"
	call	sioA_send_msg
	db	13,10,"download firmware ",0
	call	asm_download_firmware	; returns only if user bails out

; print first boot message:
12$:	call	sioA_send_msg
	db	13,10,"ROM "
romdate:: dm	__date__,0

	call	sioA_send_msg
	db	" "
romtime:: dm	__time__,0
	

; initialization of globals and statics:
	;jp	$			



; ===========================================
; 	    INCLUDE EVERYTHING
; ===========================================


;#include "VM/core-macros.s"
#include "asm/page0.s"
#include "asm/abort.s"
#include "asm/eeprom.s"		; has page0 code
;#include "asm/sio.s"
;#include "asm/i2c.s"
;#include "asm/ide.s"

#include "VM/core.s"

;#include "core/test/test_core.s"
;#include "core/test/test16.s"
;#include "core/test/test8.s"
;#include "core/test/test_str.s"

#code GSINIT
	call	sioA_send_msg
	db	13,10,"booting..",0
	ld	bc,dflt_stacks
	exx				; bc' = VM rstack 
	ld	bc,dflt_heap		; bc  = VM vstack
	;ld	de,0			; top = void
	;ld	hl,0			; aux = void
	virtual
	
#cflags  -cpu=z80-rop -v -o3 -c
;#include "src/fd.cc"
;#include "src/sio.cc"
;#include "src/BlockDevice.cc"
;#include "src/i2c.cc"
;#include "src/ide.cc"
;#include "src/File.cc"
;#include "src/main.cc"			; GSINIT should not return

#code GSINIT
	dw	panic
	dw	ERROR_END_OF_CODE	; blink code


; ________________________________________________________
#code PAGE0
page0end 	= $
page0_size 	= $ - $0000

#code ROMEND
romend 		= $
rom_size	= $ - $8000

